#!/bin/bash
configure_sudoers_integrations() {
    local new_user=$1
    local SUDOERS="/etc/sudoers"
    local perms_sudoers="$new_user ALL=(ALL) NOPASSWD: /bin/csadm package *, /bin/hamanager"
    echo "Adding sudoers for user [$new_user]"
    if grep -qE "^\s*$new_user\s+ALL\s*=\s*\(\s*ALL\s*\)\s+NOPASSWD\s*:" $SUDOERS; then
        # replace old line
        perms_sudoers=$(echo "$perms_sudoers" | sed  -e 's:/:\\\/:g')
        sed -i "s/^\s*$new_user \+ALL *= *( *ALL *) *NOPASSWD *:.*$/$perms_sudoers/g" $SUDOERS
        return
    fi
    echo "$perms_sudoers" >> $SUDOERS
}

configure_user_integrations() {
    local new_user=$1
    local old_user=$2
    local nginx_user="nginx"
    echo "Stopping uWSGI"
    systemctl stop uwsgi
    local perm_dirs="/var/run/uwsgi/ \
                     /var/log/cyops/cyops-integrations/ \
                     /opt/cyops-integrations/integrations/ \
                     /opt/cyops-integrations/.env/"
    for perm_dir in $perm_dirs; do
        setfacl -Rm u:${new_user}:rwx,u:${nginx_user}:rwx $perm_dir
        setfacl -Rdm u:${new_user}:rwx,u:${nginx_user}:rwx $perm_dir
    done

    configure_sudoers_integrations "$new_user"
    echo "Starting uWSGI"
    systemctl start uwsgi
}
############
# Main/main/MAIN starts here
#########
new_user="fsr-integrations"
old_user="nginx"
configure_user_integrations "$new_user" "$old_user"
exit 0
